/* cc -o BHB BHB.c -lm
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#define PI M_PI
#include "isihara.c"

double B[10];

void prtB(char *txt,double qB2,double q)
{
  int i;
  double p=q;

  printf("%s\n  B2 = %.9g\n",txt,B[2]*qB2);

  for (i=3; i<10; i++) {
    p*=q;
    printf("  B%d = %.8g\n",i,B[i]*p); }
}

int main(int narg,char **arg)
{
  double V,L,alpha,b;
  int i;
  char *body;

  if (narg<2) {
    fprintf(stderr,"Virial coefficients for linear molecules.\n\
hard prolate spherocylinder:\n\
  BHB L\n\
hard homonuclear diatomics:\n\
  BHB -L\n\
where L is the center-center distance, reduced by diameter sigma\n\
B2 is exact, B3..B9 are fitted\n");
    exit(0); }

  L=atof(arg[1]);

  if (L>=0) {
    body="HPS";
    if (L>1.6) printf("WARNING outside fitting range (L>1.6)\n");
    alpha=(L+1)*(L+2)/(3*L+2);
    b=L*L/((L+1)*(L+2));
    V=PI/12*(3*L+2);
    B[2]=(1+alpha*3)*V;
    B[3]=0.62500+b*(-0.16511+b*(-0.75547+b*(4.7821+b*(-24.078+b*(63.267+b*(-67.231))))));
    B[4]=0.28695+b*(-0.24608+b*(-1.0523+b*(7.5967+b*(-38.342+b*(102.31+b*(-109.76))))));
    B[5]=0.11025+b*(-0.18720+b*(-0.68857+b*(6.0110+b*(-28.494+b*(72.469+b*(-74.177))))));
    B[6]=0.03888+b*(-0.10026+b*(-0.29403+b*(3.7387+b*(-19.126+b*(52.016+b*(-57.319))))));
    B[7]=0.01302+b*(-0.04621+b*(-0.03179+b*(0.82215+b*(-2.7325+b*(3.0039)))));
    B[8]=0.00418+b*(-0.01953+b*(0.02898+b*(0.04308+b*(-0.18055))));
    B[9]=0.00132+b*(-0.00738+b*(0.01553+b*(-0.00386+b*(-0.03170)))); }
  else {
    body="HDD";
    L=-L;
    if (L>1) printf("WARNING outside fitting range (L>1)\n");
    alpha=(L+1)*(L+2)/(3*L+2-L*L*L);
    b=L*L/(L+2);
    V=PI/12*(3*L+2-L*L*L);
    B[2]=psi(L*2)*(4*PI/3);
    B[3]=0.625000+b*(-0.14973+b*(0.07646+b*(-0.64680+b*(2.6912+b*(-6.2707+b*(5.7732))))));
    B[4]=0.286950+b*(-0.22293+b*(0.13926+b*(-0.93124+b*(3.7457+b*(-8.5319+b*(7.8039))))));
    B[5]=0.110251+b*(-0.16920+b*(0.13159+b*(-0.43800+b*(1.0113+b*(-0.97363)))));
    B[6]=0.038882+b*(-0.09038+b*(0.10197+b*(-0.23483+b*(0.51835+b*(-0.47024)))));
    B[7]=0.013023+b*(-0.04015+b*(0.05586+b*(-0.06840+b*(0.06404))));
    B[8]=0.004183+b*(-0.01653+b*(0.03134+b*(-0.04980+b*(0.05004))));
    B[9]=0.001309+b*(-0.00590+b*(0.00818+b*(0.00655+b*(-0.02295))));
  }

  printf("%s L=%g alpha=%.8g V/sigma^3=%.8g b=%.8g\n",body,L,alpha,V,b);
  prtB("reduced by B2:",1/B[2],1);
  prtB("reduced by sigma^3:",1,B[2]);
  prtB("reduced by V:",1/V,B[2]/V);
}

